@@ -28,6 +28,7 @@ def get_args():
     parser.add_option('--terminate', action='store_true', dest='terminate', default=False, \
         help="Instead of creating zones delete them.")
     parser.add_option('-v', '--verbose', action='store_true', dest='verbose', default=False)
+    parser.add_option('-q', '--quite', action='store_true', dest='quite', default=False)
     parser.add_option('--debug', action='store_true', dest='debug', default=False)
 
     return parser.parse_args()
@@ -39,6 +40,8 @@ def main():
         log.setLevel(logging.DEBUG)
     elif options.verbose:
         log.setLevel(logging.INFO)
+    elif options.quite:
+        log.setLevel(logging.ERROR)
     else:
         log.setLevel(logging.WARN)
 
@@ -60,7 +63,7 @@ def main():
             if options.terminate:
                 r53zone.remove(options.dry_run)
             elif options.show:
-               print str(r53zone)
+                print str(r53zone)
             else:
                 r53zone.update(zone_file, options.dry_run)
         elif options.show or options.terminate:
