@@ -15,6 +15,8 @@ class Zone:
     or it can compare the existing Resource Records to a bind style zone file and update.
     The resource records must be sent to amazon as a xml file, the format can be found at.
     http://docs.amazonwebservices.com/Route53/latest/APIReference/index.html?API_ChangeResourceRecordSets.html
+    Route 53 has a special type called Alias that is not recognized by the dnspython library, to avoid
+    validation issues this is stored in the zone file as TXT entry and the domain name has _alias added.
     """
 
     def __init__(self, conn, zone_name):
@@ -33,10 +35,7 @@ class Zone:
             return self.zone_name + " does not exist."
 
         dnszone = self._to_dnszone()
-        tmp = tempfile.TemporaryFile()
-        dnszone.to_file(tmp)
-        tmp.seek(0)
-        return "Zone " + self.zone_name + " ID: " + self.id + "\n" + tmp.read()
+        return self._print(dnszone)
 
     def _change_alias_xml(self, action, name, rvalue):
         """Returns amazon change xml for a specific route 53 entry."""
@@ -51,11 +50,11 @@ class Zone:
 
     def _change_xml(self, action, name, rtype, ttl, values):
         """Returns amazon Change xml for an action on an entry."""
-        #Check to see if any TXT entries are a masked route53 alias
-        if rtype == 'TXT':
+        #Check to see if any entries are a masked route53 alias
+        if rtype == 'A':
             for rvalue in values:
                 if rvalue[1:7] == 'Alias ':
-                    log.debug('Interpreting TXT entry as a route53 alias.')
+                    log.info('Interpreting entry as a route53 alias.')
                     xmlout = self._change_alias_xml(action, name, rvalue)
                     if len(values) > 1:
                         values.remove(rvalue)
@@ -147,17 +146,17 @@ class Zone:
                 from_records[key].sort() #I sort both to avoid ordering issues
                 to_records[key].sort()
                 if from_records[key] != to_records[key]:
-                    log.info("Updating %s %s %s to %s" % (key[0], key[1], key[2], to_records[key]))
+                    log.warn("Updating %s %s %s to %s" % (key[0], key[1], key[2], to_records[key]))
                     updates[key] = [from_records[key], to_records[key]]
                 del from_records[key] #no-op or modify either way pull from from_records.
             else:
                 adds[key] = to_records[key]
-                log.info("Adding %s %s %s %s" % (key[0], key[1], key[2], to_records[key]))
+                log.warn("Adding %s %s %s %s" % (key[0], key[1], key[2], to_records[key]))
 
         #Anything remaining in the from_records is a delete
         for key in from_records.iterkeys():
             deletes[key] = from_records[key]
-            log.info("Removing %s %s %s %s" % (key[0], key[1], key[2], from_records[key]))
+            log.warn("Removing %s %s %s %s" % (key[0], key[1], key[2], from_records[key]))
 
         return adds, deletes, updates
 
@@ -196,6 +195,12 @@ class Zone:
                 continue
 
             rvalue = rdata.to_text()
+            if rtype == 'TXT' and rvalue[1:7] == 'Alias ':
+                log.info('Interpreting TXT entry as a route53 alias.')
+                if name[:7] == '_alias.':
+                    name = name[7:]
+                rtype = 'A'
+
             log.debug("Adding %s, type %s, ttl %d, value %s to rrecords" % (name, rtype, ttl, rvalue))
             if (name, rtype, ttl) in rrecords:
                 new_value = rrecords[(name, rtype, ttl)]
@@ -222,14 +227,13 @@ class Zone:
             rtype = str(rrecord.type)
             name = str(rrecord.name)
             ttl = str(rrecord.ttl)
-            simple_bind += "$TTL %s\n" % (ttl)
 
             if len(rrecord.resource_records) == 0 and rrecord.alias_hosted_zone_id is not None:
-                simple_bind += "%s\tIN\tTXT\t\"Alias %s %s\"\n" % (name, str(rrecord.alias_hosted_zone_id), \
-                    str(rrecord.alias_dns_name))
+                simple_bind += "%s\t%s\tIN\tTXT\t\"Alias %s %s\"\n" % ('_alias.' + name, ttl, \
+                    str(rrecord.alias_hosted_zone_id), str(rrecord.alias_dns_name))
             else:
                 for value in rrecord.resource_records:
-                    simple_bind += "%s\tIN\t%s\t%s\n" % (name, rtype, str(value))
+                    simple_bind += "%s\t%s\tIN\t%s\t%s\n" % (name, ttl, rtype, str(value))
 
             last_type = rtype
             last_name = name
@@ -247,14 +251,21 @@ class Zone:
             simple_bind += rrset_batch
             if lines > 99:
                 last_name = lname
-                last_type == ltype
+                last_type = ltype
             else:
                 break
 
-        log.debug("Simple Bind zone created from aws r53 response.\n" + simple_bind )
+        log.debug("Simple Bind zone created from aws r53 response.\n" + simple_bind + "\n")
         zone = dns.zone.from_text(simple_bind, origin=self.zone_name, relativize=False)
         return zone
 
+    def _print(self, dnszone):
+        """ Given a dnszone return its string representation. """
+        tmp = tempfile.TemporaryFile()
+        dnszone.to_file(tmp)
+        tmp.seek(0)
+        return "Zone " + self.zone_name + " ID: " + self.id + "\n" + tmp.read()
+
     def create(self, zone_file, dry_run):
         """ Create the zone and populate with settings from the passed in zone_file.
         Do nothing, report only, if dry_run is true.
@@ -263,7 +274,7 @@ class Zone:
             log.error('A zone definition already exists for zone ' + self.zone_name)
             return
         
-        log.info('Creating zone ' + self.zone_name)
+        log.warn('Creating zone ' + self.zone_name)
         if not dry_run:
             zone = self.conn.create_hosted_zone(self.zone_name)['CreateHostedZoneResponse']
             nameservers = ""
@@ -273,7 +284,7 @@ class Zone:
             self.id = zone['HostedZone']['Id'].replace('/hostedzone/', '')
             zone_xml = self._create_xml(zone_file)
             log.debug(zone_xml)
-            log.info("Adding rrsets to zone " + self.zone_name)
+            log.debug("Adding rrsets to zone " + self.zone_name)
             for xml in zone_xml:
                 change = self.conn.change_rrsets(self.id, xml)
     
@@ -283,7 +294,7 @@ class Zone:
         xmlout += self._add_change_xml('.'.join([host, self.zone_name]), rtype, ttl, [value, ])
         xmlout += self.xml_footer
         self.conn.change_rrsets(self.id, xmlout)
-        log.info("Updating %s %s %s to %s" % (host, rtype, ttl, value))
+        log.warn("Updating %s %s %s to %s" % (host, rtype, ttl, value))
 
     def exists(self):
         """Return true if the self.zone_name exists on AWS, false otherwise."""
@@ -317,19 +328,21 @@ class Zone:
                 return
 
         dnszone = dns.zone.from_file(zone_file, origin=self.zone_name, relativize=False)
+        log.info("Bind zone from local file.\n" + self._print(dnszone) + "\n")
         r53zone = self._to_dnszone()
+        log.info("Bind zone from r53.\n" + self._print(r53zone) + "\n")
 
         if dnszone == r53zone: #This compares SOA and root NS records, which are ignored in actual updates
-            log.info('Zone %s is up to date.' % self.zone_name)
+            log.warn('Zone %s is up to date.' % self.zone_name)
             return
     
         adds, deletes, updates = self._compare(r53zone, dnszone)
         changesets = self._create_changeset(adds, deletes, updates)
         if changesets is None:
-            log.info("No differences found for zone %s" % self.zone_name)
+            log.warn("No differences found for zone %s" % self.zone_name)
             return
 
-        log.info('Updating zone %s' % self.zone_name)
+        log.warn('Updating zone %s' % self.zone_name)
         if not dry_run:
             for changeset in changesets:
                 log.debug(changeset)
@@ -341,16 +354,16 @@ class Zone:
         xmlout += self._update_change_xml('.'.join([host, self.zone_name]) , rtype, ttl, existing[1], [value, ])
         xmlout += self.xml_footer
         self.conn.change_rrsets(self.id, xmlout)
-        log.info("Updating %s %s %s to %s" % (host, rtype, ttl, value))
+        log.warn("Updating %s %s %s to %s" % (host, rtype, ttl, value))
 
     def remove(self, dry_run):
         """Remove this zone from AWS."""
         if self.id == None:
             if not self.exists():
-                log.info("Zone " + self.zone_name + " doesn't exist.")
+                log.warn("Zone " + self.zone_name + " doesn't exist.")
                 return
 
-        log.info("Removing zone " + self.zone_name)
+        log.warn("Removing zone " + self.zone_name)
         if not dry_run:
             rrecords = self._get_rrecords(self._to_dnszone())
             
